/* 	BlockTemplate.java
 *	Copyright (c) 2006, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Jul 26, 2006
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;

/**
 * Simplest implementation of ContentTemplate to write blocks of code
 * to the output (used for finally, final and static blockstatements) 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.00 [26-07-2006]
 */
public class BlockTemplate extends ContentTemplate {

	private String m_blockId;
	
	/**
	 * Constructor
	 * @param blockId String a valid keyword for a block: <code>
	 * final,finally,static,synchronized,else, etc</code>
	 */
	public BlockTemplate(String blockId) {
		super();
		m_blockId=blockId;
	}

	/**
	 * {@inheritDoc}
	 */
	public void write(PrintWriter writer, int indent) {
		String spc=TypeWriterHelper.spaces(indent*4);
		if (!"finally".equals(m_blockId) && !"else".equals(m_blockId)) writer.print(spc); else writer.print(" ");
		writer.println(m_blockId + " {");
		super.write(writer, indent+1);
		writer.print(spc);
		writer.println("}");
	}
}
