/* 	KeyWords.java
 *		Copyright (c) 2003, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * @#History 
 * [When] 		[Who] 				[What] 
 * Aug 19, 2006 	dennis@brains2b.nl	Added isMethodReserved to not overwrite object methods
 */
package org.brains2b.typewriter;


/** Class that implements single static functions <code>isReserved</code>
 * and <code>isMethodReserved</code> which checks if a <code>String</code> 
 * is a Java reserved word.
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [Oct 29, 2003]
 */
public class KeyWords {
	
	/** list of Java reserved words */
	private final static String[] KEYWORDS= new String[] {
		"abstract",
		"double",
		"int",
		"strictfp",
		"boolean",
		"else",
		"interface",
		"super",
		"break",
		"extends",
		"long",
		"switch",
		"byte",
		"final",
		"native",
		"synchronized",
		"case",
		"finally",
		"new",
		"this",
		"catch",
		"float",
		"package",
		"throw",
		"char",
		"for",
		"private",
		"throws",
		"class",
		"goto",
		"protected",
		"transient",
		"const",
		"if",
		"public",
		"try",
		"continue",
		"implements",
		"return",
		"void",
		"default",
		"import",
		"short",
		"volatile",
		"do",
		"instanceof",
		"static",
		"while"
	};
	
	/**
	 * Default Constructor
	 * <p>constructor made private to prevent instantiation
	 */
	private KeyWords() {
		super();
	}
	
	/**
	 * check if a String is a Java reserved word/keyword 
	 * @param name String, the value to check
	 * @return boolean, true if the String is a keyword, otherwise false, also if the
	 * name is null.
	 */
	public static boolean isReserved(String name) {
		if (name==null) return false;
		for (int i=0;i<KEYWORDS.length;i++) {
			if (KEYWORDS[i].equals(name)) return true;
		}
		return false;
	}

	/**
	 * check if a String is a Java reserved method
	 * <p>Only one will return true at the moment getClass()
	 * @param name String, the value to check
	 * @return boolean, true if the String is a reserverd method, otherwise false, also if the
	 * name is null.
	 */
	public static boolean isMethodReserved(String name) {
		if ("getClass".equals(name)) return true;
		return false;
	}
}
