/* 	Literal.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Oct 29, 2003
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.typewriter;

/**Literal is a simple wrapper class which makes sure that a value supplied
 * is written into the template as is for both String and char.
 * <p>A String passed to Literal would be used in the source as "String", a char
 * passed to Literal would be used as 'c'har, All other Strings and characters will
 * be threated as being part of the code in one way or another 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.20 [27-08-2004]
 */
public class Literal {
	private String value;
	private char c;
	
	/**
	 * Constructor
	 * @param value String, the value which should be represented Literally 
	 */
	public Literal(String value) {
		this.value=value;	
	}

    /**
     * Constructor
     * @param c char, the char which should be represented Literally 
     */
    public Literal(char c) {
        this.c=c;
    }
	
	/**
	 * Overwritten to represent a String between apostrofes and 
	 * a char between quotes
	 */
	public String toString() {
        StringBuffer sb=new StringBuffer();
		if (c!=0) {
		    sb.append("\'").append(c).append("\'");
        } else if (value!=null) {
            sb.append("\"").append(value).append("\"");
        } else {
            return null;
        }
        return sb.toString();
	}

}
