package org.brains2b.typewriter;

import java.io.PrintWriter;

/** 
 * Create a loop element in a method
 * <UL>A loop can either be
 * <LI>DO
 * <LI>WHILE
 * <LI>FOR
 * </UL>
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [revision date]
 */
public class LoopTemplate extends ContentTemplate {
	
	/** constant to define a do while loop*/
	public final static int DO=0x01;
	/** constant to define a while loop*/
	public final static int WHILE=0x02;
	/** constant to define a for loop*/
	public final static int FOR=0x03;
	
	private String m_condition;	
	private int m_type=FOR;
	
	/**
	 * Constructor
	 * @param type int, one of the loop types (DO|WHILE|FOR)
	 * @param condition String, a complete condition for this loop i.e.
	 * <code>int i=0;i<x.size();i++</code> in a for loop.
	 */
	public LoopTemplate(int type,String condition) {
		super();
		m_type=type;
		m_condition=condition;
	}
	
	/* (non-Javadoc)
	 * @see org.brains2b.typewriter.Template#write(java.io.PrintWriter, int)
	 */
	public void write(PrintWriter writer, int indent) {
		String spc=TypeWriterHelper.spaces(indent*4);
		writer.print(spc);
		switch (m_type) {
			case DO:
				writer.println("do {");
				break;
			case WHILE:
				writer.print("while (");
				writer.print(m_condition) ;
				writer.println(") {");
				break;
			case FOR:
				writer.print("for (");
				writer.print(m_condition) ;
				writer.println(") {");
		}
		
		super.write(writer,indent+1);
		writer.print(spc);
		switch( m_type) {
			case DO:
				writer.print("} while (");
				writer.print(m_condition) ;
				writer.println(")");
				break;
			default:
				writer.println("}");
		}
		
	}

	/**
	 * set the condition.
	 * @param condition String, the condition for this loop
	 */
	public void setCondition(String condition) {
		m_condition = condition;
	}

}
