/* 	Template.java
 *	Copyright (c) 2002-2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 12-09-2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 10/28/2002	dennis@brains2b.nl	Added methods: addImport, getObjectName
 * 27-08-2004   dennis@brains2b.nl  deprecated getObjectName(String)
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;

/** Definition of a Template used both for writing the Class and the containing elements
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [28-10-2002]
 */
public interface Template {
	
	/**
	 * write the template information to the PrintWriter at the given
	 * indention.
	 * <p>Indention is multiplied by <code>4</code> to supply the spaces
	 * for the indention.
	 * @param writer PrintWriter, an initialized PrintWriter to write this
	 * element of the Class to
	 * @param indent int, the indention for this element. 
	 * <p>Value is multiplied by four to give the number of spaces used for
	 * indention
	 */
	public void write(PrintWriter writer, int indent);
	
	/**
	 * get the direct parent of this element.
	 * <p>This can be the class, method, try element or loop elements
	 * @return Template, the parent of this element
	 */
	public Template getParent();
	
	/**
	 * set the direct parent of this element
	 * <p>The parent can be the class, method, try element or loop elements
	 * @param t Template, the parent of this element
	 */
	public void setParent(Template t);
	
	/**
	 * add an String as import to the top class of this element
	 * @param name String, the fully qualified class name. Could be used
	 * to enter the complete package by adding <code>*</code> 
	 */
	public void addImport(String name);
	
}
