/* 	TryTemplate.java
 *	Copyright (c) 2002-2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sep 4, 2002
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]					[What]
 * Feb 2, 2004	dennis@brains2b.nl		Small fixes for writing Exception
 * 										Added comments		
 * Jul 26,2006	dennis@brains2b.nl		Added support for finally statement
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/** 
 * Create a Try statement.
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.12 [26-07-2006]
 */
public class TryTemplate extends  ContentTemplate {

	private List m_exceptions;
	private BlockTemplate m_final;;
	
	/**
	 * Constructor for TryTemplate.
	 * @param exceptionClass Class, the class for the Exception this
	 * try should catch. You can add more Exceptions by using createException(String)
	 * @see #createException
	 */
	public TryTemplate(Class exceptionClass) {
		this(exceptionClass.getName());
	}
	
	/**
	 * Constructor for TryTemplate.
	 * @param exceptionClassName String, a String representing the complete class name, including package of the
	 * Exception this try should catch. You can add more Exceptions by using createException(String)
	 * @see #createException
	 */
	public TryTemplate(String exceptionClassName) {
		ExceptionTemplate et=new ExceptionTemplate(exceptionClassName);
		et.setParent(getParent());
		m_exceptions = new ArrayList();
		m_exceptions.add(et);
	}	
	
	/**
	 * write the TryStatement to the PrintWriter.
	 * @see org.brains2b.typewriter.Template#write(PrintWriter, int)
	 */
	public void write(PrintWriter writer, int indent) {
		String spc=TypeWriterHelper.spaces(indent*4);
		writer.print(spc);
		writer.println("try {");
		
		super.write(writer,++indent);
		for (int i=0;i<m_exceptions.size();i++) {
			ExceptionTemplate et= (ExceptionTemplate) m_exceptions.get(i);
			et.write(writer,indent-1);
			writer.print(spc);
			if (i==m_exceptions.size()-1) {
				if (m_final==null) writer.println("}"); else writer.print("}");
			} 
		}
		if (m_final!=null) {
			m_final.write(writer,indent-1);
		}
		
		
	}

	/**
	 * get the Exception this Try statement was created with
	 * @return ExceptionTemplate
	 */
	public ExceptionTemplate getDefaultException() {
		return (ExceptionTemplate) m_exceptions.get(0);
	}

	/**
	 * create a new Exception for this Try statement. This Exception will be
	 * added in a list of catch Statements for this try. Exceptions will be
	 * added to the catch blocks in order they where added. Starting with the 
	 * the one this Try statement was created for.
	 * @param exceptionClassName String, the fully qualified class name of this Exception
	 * @return ExceptionTemplate
	 */
	public ExceptionTemplate createException(String exceptionClassName) {
		ExceptionTemplate et=new ExceptionTemplate(exceptionClassName);
		et.setParent(getParent());
		m_exceptions.add(et);
		return et;
	}
	
	/**
	 * create a finally block for this Exception
	 * @return BlockTemplate, the <code>finally</code> block
	 */
	public BlockTemplate createFinally() {
		BlockTemplate ft=new BlockTemplate("finally");
		m_final=ft;
		return ft;
	}

}
