/* 	ClassTemplateTest.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Aug 27, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Mar 14, 2005	dennis@brains2b.nl		Added tests for renaming and alike constructors
 * Oct 19, 2005 dennis@brains2b.nl      Removed unused variables
 * Apr 11, 2006 	dennis@brains2b.nl	Write an empty line between package and import
 */
package test.org.brains2b.typewriter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Modifier;

import junit.framework.TestCase;

import org.brains2b.typewriter.ClassTemplate;
import org.brains2b.typewriter.CommentTemplate;
import org.brains2b.typewriter.Literal;
import org.brains2b.typewriter.MethodTemplate;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.1 [19-10-2005]
 */
public class ClassTemplateTest extends TestCase {
	public final static String TEST_CREATE_1="import org.brains2b.typewriter.ContentTemplate;\n\npublic class Test extends ContentTemplate implements Clonable, Runnable {\n\n}\n";
	public final static String TEST_CREATE_2="package org.brains2b.test;\n\nimport org.brains2b.typewriter.ContentTemplate;\n\npublic class Test extends ContentTemplate {\n\n}\n";
	public final static String TEST_INNER="\npublic class Test {\n\n    public class Inner implements Clonable {\n\n    }\n\n}\n";
	public final static String TEST_COMMENT="/*\n * This is a Test class\n */\n\npublic class Test {\n\n}\n";
	public final static String TEST_CONSTRUCTOR_1="\npublic class Test {\n\n    public Test(String arg0) {\n        String id=arg0;\n    }\n\n}\n";
	public final static String TEST_CONSTRUCTOR_2="\npublic class Test {\n\n    public Test(String id) {\n        String tid=id;\n    }\n\n}\n";
	public final static String TEST_CONSTRUCTOR_DOUBLE="\npublic class Test {\n\n    public Test(int arg0) {\n        int id=arg0;\n    }\n\n    public Test(String arg0) {\n        String id=arg0;\n    }\n\n}\n";
	public final static String TEST_CONSTRUCTOR_RENAME="\npublic class RenameTest {\n\n    public RenameTest(String arg0) {\n        String id=arg0;\n    }\n\n}\n";
	public final static String TEST_METHOD_1="\npublic class Test {\n\n    public void setId(int arg0) {\n        this.id=arg0;\n    }\n\n}\n";
	public final static String TEST_METHOD_2="\npublic class Test {\n\n    public String getId() {\n        return id;\n    }\n\n}\n";
	public final static String TEST_METHOD_3="\npublic class Test {\n\n    protected void setId(int arg0) {\n        this.id=id;\n    }\n\n}\n";
	public final static String TEST_METHOD_4="\npublic class Test {\n\n    private boolean setValues(int id, String name) {\n        this.id=id;\n        this.name=name;\n        return true;\n    }\n\n}\n";
	public final static String TEST_MEMBER_1="\npublic class Test {\n\n    private int id = 0;\n}\n";
	public final static String TEST_MEMBER_2="\npublic class Test {\n\n    private String name = \"MyName\";\n}\n";
	public final static String TEST_MEMBER_3="\npublic class Test {\n\n    private int id;\n}\n";
	public final static String TEST_IMPORT="import javax.swing.JFrame;\n\npublic class Test {\n\n}\n";
	public final static String TEST_PACKAGE="package org.brains2b.test;\n\npublic class Test {\n\n}\n";
	
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(ClassTemplateTest.class);
	}

	/**
	 * Constructor for ClassTemplateTest.
	 * @param arg0
	 */
	public ClassTemplateTest(String arg0) {
		super(arg0);
	}

	public void testConstructor1() {
		ClassTemplate ct=new ClassTemplate("Test","org.brains2b.typewriter.ContentTemplate",new String[] { "Runnable","Clonable" });
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_CREATE_1,sw.toString());

	}
	
	public void testConstructor2() {
		ClassTemplate ct=new ClassTemplate("org.brains2b.test","Test","org.brains2b.typewriter.ContentTemplate",null);
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_CREATE_2,sw.toString());

	}
	
	public void testWriteClass() {
		ClassTemplate ct=new ClassTemplate("org.brains2b.test","MyClass",null,null);
		//add a comment
		ct.createComment().addText("This is a TypeWriter test Class");
		//add members
		ct.createMember("int","id","0");
		ct.createMember("String","name");
		//add a constructor
		MethodTemplate cons=ct.createConstructor(new String[][]{{"int","String"},{"id","name"}});
		cons.addText("setId(id);");
		cons.addText("this.name=name;");
		//add methods
		MethodTemplate m=ct.createMethod("setName",new String[]{"String"});
		m.addText("name=arg0;");
		MethodTemplate m_id=ct.createMethod("getId","int",null);
		m_id.addText("return id;");
		
		// the short form
		ct.createMethod("getName","String",null).addText("return name;");
		
		MethodTemplate m_sid=ct.createMethod(Modifier.PROTECTED,"setId",null,new String[][]{{"int"},{"id"}});
		m_sid.addText("this.id=id;");
		
		try {
			ct.writeClass(System.getProperty("java.io.tmpdir"));
			comparefiles(new File("test/org/brains2b/test/MyClass.java"),ct.getCompleteFile());
            ct.getCompleteFile().delete();
		} catch (IOException iex) {
			assertTrue(iex.getMessage(),false);
		}
		
	}

	public void testCreateInnerClass() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.createInnerClass("Inner",null,new String[] {"Clonable"});
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_INNER,sw.toString());
	}

	public void testCreateComment() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		CommentTemplate t=ct.createComment();
		t.addText("This is a Test class");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_COMMENT,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createConstructor(String[])
	 */
	public void testCreateConstructorStringArray() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createConstructor(new String[] {"String"} );
		t.addText("String id=arg0;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_CONSTRUCTOR_1,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createConstructor(String[][])
	 */
	public void testCreateConstructorStringArrayArray() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createConstructor(new String[][] {{"String"},{"id"}} );
		t.addText("String tid=id;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_CONSTRUCTOR_2,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createConstructor(String[])
	 */
	public void testCreateDoubleConstructor() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createConstructor(new String[] {"String"} );
		t.addText("String id=arg0;");
		MethodTemplate t2=ct.createConstructor(new String[] {"int"} );
		t2.addText("int id=arg0;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_CONSTRUCTOR_DOUBLE,sw.toString());
	}
	/*
	 * Class under test for MethodTemplate createConstructor(String[])
	 */
	public void testRenameConstructor() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createConstructor(new String[] {"String"} );
		t.addText("String id=arg0;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.setName("RenameTest");
		ct.write(pw,0);
		assertEquals(TEST_CONSTRUCTOR_RENAME,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createMethod(String, String[])
	 */
	public void testCreateMethodStringStringArray() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createMethod("setId",new String[]{"int"} );
		t.addText("this.id=arg0;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_METHOD_1,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createMethod(String, String, String[])
	 */
	public void testCreateMethodStringStringStringArray() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createMethod("getId","String",null);
		t.addText("return id;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_METHOD_2,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createMethod(int, String, String, String[])
	 */
	public void testCreateMethodintStringStringStringArray() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createMethod(Modifier.PROTECTED,"setId",null,new String[] {"int"});
		t.addText("this.id=id;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_METHOD_3,sw.toString());
	}

	/*
	 * Class under test for MethodTemplate createMethod(int, String, String, String[][])
	 */
	public void testCreateMethodintStringStringStringArrayArray() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		MethodTemplate t=ct.createMethod(Modifier.PRIVATE,"setValues","boolean",new String[][] {{"int","String"},{"id","name"}});
		t.addText("this.id=id;");
		t.addText("this.name=name;");
		t.addText("return true;");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_METHOD_4,sw.toString());
	}

	/*
	 * Class under test for MemberTemplate createMember(String, String, Object)
	 */
	public void testCreateMemberStringStringObject() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.createMember("int","id","0");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_MEMBER_1,sw.toString());
		
	}
	
	/*
	 * Class under test for MemberTemplate createMember(String, String, Object)
	 */
	public void testCreateMemberStringStringObject2() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.createMember("String","name",new Literal("MyName"));
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_MEMBER_2,sw.toString());
		
	}

	/*
	 * Class under test for MemberTemplate createMember(String, String, Object, boolean)
	 */
	public void testCreateMemberStringStringObjectboolean() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.createMember("int","id","0",true);
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_MEMBER_1,sw.toString());
	}

	/*
	 * Class under test for MemberTemplate createMember(String, String)
	 */
	public void testCreateMemberStringString() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.createMember("int","id");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_MEMBER_3,sw.toString());
	}

	public void testAddImport() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.addImport("javax.swing.JFrame");
		ct.addImport("javax.swing.JFrame"); // should only be added once
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_IMPORT,sw.toString());
	}

	public void testPackage() {
		ClassTemplate ct=new ClassTemplate("Test",null,null);
		ct.setPackage("org.brains2b.test");
		StringWriter sw=new StringWriter();
		PrintWriter pw=new PrintWriter(sw);
		ct.write(pw,0);
		assertEquals(TEST_PACKAGE,sw.toString());
	}
	
	private void comparefiles(File left, File right) {
		try {
			BufferedReader lbr=new BufferedReader(new FileReader(left));
			BufferedReader rbr=new BufferedReader(new FileReader(right));
			String lline=lbr.readLine();
			String rline=rbr.readLine();
			int count=0;
			while (lline!=null) {
				count++;
				if (rline!=null) {
					if (lline.indexOf("@since")==-1) { // ignore timestamp in since for string compare
						assertEquals("difference at line:" + count,lline,rline);
					} else {
						assertEquals(lline.indexOf("@since"),rline.indexOf("@since"));
					}
				} else {
					assertNull("difference at line:" + count + ":" + "could not find it in compared file");
				}
				lline=lbr.readLine();
				rline=rbr.readLine();
			}
			assertNull("compared file still contains data",rbr.readLine());
			lbr.close();
			rbr.close();
		} catch (Exception ex) {
			assertTrue(ex.getMessage(),false);
		}
	}
}
