/* 	ContentTemplateTest.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Aug 27, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package test.org.brains2b.typewriter;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Vector;

import javax.swing.JFrame;

import org.brains2b.typewriter.ClassTemplate;
import org.brains2b.typewriter.ContentTemplate;
import org.brains2b.typewriter.IfTemplate;
import org.brains2b.typewriter.Literal;
import org.brains2b.typewriter.LoopTemplate;
import org.brains2b.typewriter.TryTemplate;

import junit.framework.TestCase;

/** Unit test for org.brains2b.typewriter.ContentTemplate
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [27-08-2004]
 * @see org.brains2b.typewriter.ContentTemplate
 */
public class ContentTemplateTest extends TestCase {
	public final static String WRITE_TEST="int x=0;\nif(x<10) {\n}\n";
	public final static String INSTANCE_1="JFrame frm = new JFrame();";
	public final static String INSTANCE_2="JFrame frm = new JFrame( \"title\" );";
	public final static String INSTANCE_3="JFrame frm = new JFrame( \"title\", gc );";
	public final static String IMPORT_TEST="import javax.swing.JFrame;\n\npublic class Test {\n\n}\n";	
	/* class is abstract so we need a Dummy */
	public class DummyTemplate extends ContentTemplate {
		public DummyTemplate() {
			super();
		}
	}
	
    /**
     * Constructor for ContentTemplateTest.
     * @param arg0
     */
    public ContentTemplateTest(String arg0) {
        super(arg0);
    }

    public static void main(String[] args) {
        junit.swingui.TestRunner.run(ContentTemplateTest.class);
    }

    public void testAddText() {
    	final String test="int x=0;";
    	DummyTemplate dt=new DummyTemplate();
    	dt.addText(test);
    	assertEquals(1,dt.getContent().size());
    	assertEquals(test,dt.getContent().get(0));
    }

    public void testCreateFor() {
    	final String condition="int x=0;i<10;i++";
    	DummyTemplate dt=new DummyTemplate();
    	LoopTemplate test=dt.createFor(condition);
    	assertEquals(1,dt.getContent().size());
    	assertEquals(test,dt.getContent().get(0));
    }

    public void testCreateWhile() {
    	final String condition="x<10";
    	DummyTemplate dt=new DummyTemplate();
    	LoopTemplate test=dt.createWhile(condition);
    	assertEquals(1,dt.getContent().size());
    	assertEquals(test,dt.getContent().get(0));
    }

    public void testCreateIf() {
    	final String condition="x<10";
    	DummyTemplate dt=new DummyTemplate();
    	IfTemplate test=dt.createIf(condition);
    	assertEquals(1,dt.getContent().size());
    	assertEquals(test,dt.getContent().get(0));
    }

    public void testCreateTry() {
    	final String exception="RuntimeException";
    	DummyTemplate dt=new DummyTemplate();
    	TryTemplate test=dt.createTry(exception);
    	assertEquals(1,dt.getContent().size());
    	assertEquals(test,dt.getContent().get(0));
    }

    public void testWrite() {
    	final String test="int x=0;";
    	DummyTemplate dt=new DummyTemplate();
    	dt.addText(test);
    	dt.createIf("x<10");
    	assertEquals(2,dt.getContent().size());
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	dt.write(pw,0);
    	pw.close();
    	assertEquals(WRITE_TEST,sw.toString());
    	
    }

    public void testParent() {
    	final ClassTemplate ct=new ClassTemplate("Test",null,null);
    	DummyTemplate dt=new DummyTemplate();
    	dt.setParent(ct);
    	assertEquals(ct,dt.getParent());
    }

    public void testContent() {
    	DummyTemplate dt=new DummyTemplate();
    	assertNotNull(dt.getContent());
    	assertTrue(dt.getContent().isEmpty());
    	dt.addText("int x=0;");
    	assertTrue(!dt.getContent().isEmpty());
    	Vector v=new Vector();
    	v.add("int y=0;");
    	dt.setContent(v);
    	assertEquals(1,dt.getContent().size());
    	assertEquals("int y=0;",dt.getContent().get(0));
    	dt.setContent(null);
    	assertNotNull(dt.getContent());
    	assertTrue(dt.getContent().isEmpty());
    }
    
    /*
     * Test for String makeInstance(Class, String)
     */
    public void testMakeInstanceClassString() {
    	DummyTemplate dt=new DummyTemplate();
    	dt.makeInstance(JFrame.class.getName(),"frm",null);
    	assertEquals(1,dt.getContent().size());
    	assertEquals(INSTANCE_1,dt.getContent().get(0));
    }

    /*
     * Test for String makeInstance(Class, String, Object)
     */
    public void testMakeInstanceClassStringObject() {
    	DummyTemplate dt=new DummyTemplate();
    	dt.makeInstance(JFrame.class.getName(),"frm",new Literal("title"));
    	assertEquals(1,dt.getContent().size());
    	assertEquals(INSTANCE_2,dt.getContent().get(0));
    }

    /*
     * Test for String makeInstance(String, String, Object)
     */
    public void testMakeInstanceStringStringObject() {
    	DummyTemplate dt=new DummyTemplate();
    	dt.makeInstance("javax.swing.JFrame","frm",new Literal("title"));
    	assertEquals(1,dt.getContent().size());
    	assertEquals(INSTANCE_2,dt.getContent().get(0));
    }

    /*
     * Test for String makeInstance(Class, String, Object[])
     */
    public void testMakeInstanceClassStringObjectArray() {
    	DummyTemplate dt=new DummyTemplate();
    	dt.makeInstance(JFrame.class.getName(),"frm",new Object[] {new Literal("title"),"gc"});
    	assertEquals(1,dt.getContent().size());
    	assertEquals(INSTANCE_3,dt.getContent().get(0));
    }

    /*
     * Test for String makeInstance(String, String, Object[])
     */
    public void testMakeInstanceStringStringObjectArray() {
    	DummyTemplate dt=new DummyTemplate();
    	dt.makeInstance("javax.swing.JFrame","frm",new Object[] {new Literal("title"),"gc"});
    	assertEquals(1,dt.getContent().size());
    	assertEquals(INSTANCE_3,dt.getContent().get(0));
    }

    public void testAddImport() {
    	final ClassTemplate ct=new ClassTemplate("Test",null,null);
    	DummyTemplate dt=new DummyTemplate();
    	dt.setParent(ct);
    	dt.addImport("javax.swing.JFrame");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	ct.write(pw,0);
    	pw.close();
    	assertEquals(IMPORT_TEST,sw.toString());
    }


}
