/* 	IfTemplateTest.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Aug 27, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package test.org.brains2b.typewriter;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.brains2b.typewriter.IfTemplate;
import org.brains2b.typewriter.TypeWriterException;

import junit.framework.TestCase;

/** Unit test for org.brains2b.typewriter.IfTemplate
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [27-08-2004]
 * @see org.brains2b.typewriter.IfTemplate
 */
public class IfTemplateTest extends TestCase {
	public final static String IFT_TEST="if(x>1) {\n    x=0;\n}\n";
	public final static String IFT_TEST2="if(x>1 || (y>0 && y<25)) {\n}\n";
	public final static String IFT_ELSE="if(x>=1) {\n    x=0;\n} else {\n    x=-1;\n}\n";
    public final static String IFT_ELSE2="if(x>=1) {\n} else if(x==0) {\n}\n";
    public final static String IFT_CONDITION="if(x<1) {\n    x=0;\n}\n";
	/**
     * Constructor for IfTemplateTest.
     * @param arg0
     */
    public IfTemplateTest(String arg0) {
        super(arg0);
    }

    public static void main(String[] args) {
        junit.swingui.TestRunner.run(IfTemplateTest.class);
    }

    /*
     * Test for void IfTemplate(String)
     */
    public void testIfTemplateString() {
    	IfTemplate ift=new IfTemplate("x>1");
    	ift.addText("x=0;");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	ift.write(pw,0);
    	pw.close();
    	assertEquals(IFT_TEST,sw.toString());
    }
    
     /*
     * Test for void IfTemplate(String)
     */
    public void testIfTemplateNull() {
    	IfTemplate ift=new IfTemplate("");
    	ift.addText("x=0;");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	try {
    		ift.write(pw,0);
    		assertTrue("Should not get here",false);
    	} catch (TypeWriterException twex) {
    		assertTrue(true);
    		assertEquals("cannot write an IfTemplate without a condition",twex.getMessage());
    	}
    	pw.close();
    }
    
     public void testIfTemplateString2() {
    	IfTemplate ift=new IfTemplate("x>1 || (y>0 && y<25)");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	ift.write(pw,0);
    	pw.close();
    	assertEquals(IFT_TEST2,sw.toString());
    }
    
    public void testCreateElse() {
    	IfTemplate ift=new IfTemplate("x>=1");
    	ift.addText("x=0;");
    	ift.createElse().addText("x=-1;");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	ift.write(pw,0);
    	pw.close();
    	assertEquals(IFT_ELSE,sw.toString());
    }
    
    public void testCreateElseIf() {
    	IfTemplate ift=new IfTemplate("x>=1");
    	ift.createElse().createIf("x==0");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	ift.write(pw,0);
    	pw.close();
    	assertEquals(IFT_ELSE2,sw.toString());
    	
    }

    public void testSetCondition() {
    	IfTemplate ift=new IfTemplate("x>1");
    	ift.addText("x=0;");
    	ift.setCondition("x<1");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	ift.write(pw,0);
    	pw.close();
    	assertEquals(IFT_CONDITION,sw.toString());
    }

}
