package test.org.brains2b.typewriter;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Modifier;

import junit.framework.TestCase;

import org.brains2b.typewriter.Literal;
import org.brains2b.typewriter.MemberTemplate;
import org.brains2b.typewriter.TypeWriterException;

/** 
 * Unit test for org.brains2b.typewriter.MemberTemplate
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.00 [09/11/02]
 * @see org.brains2b.typewriter.MemberTemplate
 */
public class MemberTemplateTest extends TestCase {

	private PrintWriter m_writer;
	private StringWriter m_string;

	/**
	 * Constructor for MemberTemplateTest.
	 * @param arg0
	 */
	public MemberTemplateTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(MemberTemplateTest.class);
	}

	/**
	 * @see TestCase#setUp()
	 */
	public void setUp() {
		m_string=new StringWriter();
		m_writer=new PrintWriter(m_string);
	}

	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		m_writer.close();
	}

	/*
	 * Test for void MemberTemplate(String, String)
	 */
	public void testMemberTemplateStringString() {
		MemberTemplate mem=new MemberTemplate("int","name");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private int name;", m_string.toString());
		
		try {
			mem=new MemberTemplate("int",null);
			assertTrue(false);
		} catch (TypeWriterException twex) {
			assertTrue(twex.getMessage(),true);
		}
		
		try {
			mem=new MemberTemplate("","name");
			assertTrue(false);
		} catch (TypeWriterException twex) {
			assertTrue(twex.getMessage(),true);
		}
		
	}

	/*
	 * Test for void MemberTemplate(String, String, Object)
	 */
	public void testMemberTemplateStringStringObject() {
		MemberTemplate mem=new MemberTemplate("int","name","water");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private int name = water;", m_string.toString());
	}
		
	public void testMemberTemplateStringStringObject2() {
		MemberTemplate mem=new MemberTemplate("String","name",new Literal("water"));
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private String name = \"water\";", m_string.toString());
	}
	
	public void testMemberTemplateStringStringObject3() {
		MemberTemplate mem=new MemberTemplate("int","name",null);
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private int name;", m_string.toString());
		
	}

	/*
	 * Test for void MemberTemplate(int, String, String)
	 */
	public void testMemberTemplateIStringString() {
		MemberTemplate mem=new MemberTemplate(Modifier.PUBLIC,"int","name");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("public int name;", m_string.toString());
	}

	public void testMemberTemplateIStringString2() {
		MemberTemplate mem=new MemberTemplate(Modifier.PRIVATE + Modifier.STATIC,"int","name");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private static int name;", m_string.toString());
	}

	public void testMemberTemplateIStringString3() {
		MemberTemplate mem=new MemberTemplate(0,"int","name");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("int name;", m_string.toString());
	}

	/*
	 * Test for void MemberTemplate(int, String, String, Object)
	 */
	public void testMemberTemplateIStringStringObject() {
		MemberTemplate mem=new MemberTemplate(Modifier.PRIVATE + Modifier.FINAL + Modifier.STATIC,"int","name","0");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private final static int name = 0;", m_string.toString());
	}

	public void testSetInitialValue() {
		MemberTemplate mem=new MemberTemplate(Modifier.PRIVATE,"int","name");
		mem.setInitialValue("later");
		mem.write(m_writer,0);
		m_writer.flush();
		assertEquals("private int name = later;", m_string.toString());
	}


	public void testSetInitialValue2() {
		MemberTemplate mem=new MemberTemplate(Modifier.PRIVATE + Modifier.FINAL + Modifier.STATIC,"int","name");
		try {
			mem.write(m_writer,0);
			assertTrue(false);
		} catch (TypeWriterException twex) {
			assertTrue(twex.getMessage(),true);
		}
		m_writer.flush();
	}


}
