package test.org.brains2b.typewriter;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Modifier;

import org.brains2b.typewriter.Literal;
import org.brains2b.typewriter.MethodTemplate;

import junit.framework.TestCase;

/** Unit test for org.brains2b.typewriter.MethodTemplate
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [20-08-2004]
 * @see org.brains2b.typewriter.MethodTemplate
 */
public class MethodTemplateTest extends TestCase {
	public final static String METHOD_CONSTRUCT_1="public int getId() {\n}\n";
	public final static String METHOD_CONSTRUCT_2="void setId(int arg0) {\n}\n";
	public final static String METHOD_CONSTRUCT_3="protected static void setId(int id) {\n    this.id=id;\n}\n";
	public final static String METHOD_CONSTRUCT_4="private static void setName(String firstName, String secondName) {\n}\n";
	public final static String METHOD_THROWS_1="public int getId() throws NullPointerException {\n}\n";
	public final static String METHOD_THROWS_2="public int getId() throws NullPointerException, SAXException {\n}\n";
	public final static String METHOD_INVOKE_1="public int getId() {\n    auto.getAutoId( \"literal\", Auto.NON_LITERAL );\n}\n";
	public final static String METHOD_INVOKE_2="public int getId() {\n    auto.getAutoId( \"literal\" );\n}\n";
	public final static String METHOD_ISCONSTR="public TestClass(int arg0, String arg1) {\n}\n";
    /**
     * Constructor for MethodTemplateTest.
     * @param arg0
     */
    public MethodTemplateTest(String arg0) {
        super(arg0);
    }

    public static void main(String[] args) {
        junit.swingui.TestRunner.run(MethodTemplateTest.class);
    }

    /*
     * Test for void MethodTemplate(ClassTemplate, int, String, String, String[])
     */
    public void testMethodTemplateConstructor() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[])null);
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_CONSTRUCT_1,sw.toString());
    	
    }
    
    public void testMethodTemplateConstructorTwo() {
    	MethodTemplate m=new MethodTemplate(null,0,"setId",null,new String[] {"int"});
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_CONSTRUCT_2,sw.toString());	
    }
    
    public void testMethodTemplateConstructorThree() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PROTECTED+Modifier.STATIC,"setId",null,new String[][] {{"int"},{"id"}});
    	m.addText("this.id=id;");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_CONSTRUCT_3,sw.toString());
    	
    }
    
    public void testMethodTemplateConstructorFour() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PRIVATE+Modifier.STATIC,"setName",null,new String[][] {{"String","String"},{"firstName","secondName"}});
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_CONSTRUCT_4,sw.toString());
    	
    }

    public void testAddException() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[])null);
    	m.addException("java.lang.NullPointerException");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_THROWS_1,sw.toString());
    }
    
    public void testAddExceptionTwo() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[])null);
    	m.addException("java.lang.NullPointerException");
    	m.addException("org.xml.sax.SAXException");
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_THROWS_2,sw.toString());
    }

    /*
     * Test for void invokeMethod(String, String, String)
     */
    public void testInvokeMethodStringStringString() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[])null);
    	m.invokeMethod("auto","getAutoId",new Object[] {new Literal("literal"),"Auto.NON_LITERAL"});
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_INVOKE_1,sw.toString());
    }

    /*
     * Test for void invokeMethod(String, String, Object[])
     */
    public void testInvokeMethodStringStringObjectArray() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[])null);
    	m.invokeMethod("auto","getAutoId",new Literal("literal"));
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_INVOKE_2,sw.toString());
    }
   
    public void testSetConstructor() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"TestClass",null,new String[] {"int","String"});
    	m.setConstructor(true);
    	assertTrue(m.isConstructor());
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	m.write(pw,0);
    	pw.close();
    	assertEquals(METHOD_ISCONSTR,sw.toString());
    }

    public void testGetName() {
    	MethodTemplate m=new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[])null);
    	assertEquals("getId",m.getName());
    }

    public void testCompareTo() {
    	MethodTemplate m  = new MethodTemplate(null,Modifier.PUBLIC,"getId","int",(String[]) null);
    	MethodTemplate m2 = new MethodTemplate(null,Modifier.PRIVATE,"setId",null,new String[] {"int"});
    	MethodTemplate m3 =new MethodTemplate(null,Modifier.PUBLIC,"getName","String",(String[]) null);
    	assertEquals(-1,m.compareTo(m2));
    	assertEquals(-5,m.compareTo(m3));
    	assertEquals(1,m2.compareTo(m));
    	assertEquals(1,m2.compareTo(m3));
    	assertEquals(5,m3.compareTo(m));
    	assertEquals(-1,m3.compareTo(m2));
    	
    }

}
