/* 	TryTemplateTest.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 2, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package test.org.brains2b.typewriter;

import java.io.PrintWriter;
import java.io.StringWriter;

import junit.framework.TestCase;

import org.brains2b.typewriter.ExceptionTemplate;
import org.brains2b.typewriter.TryTemplate;

/** Unit test for org.brains2b.typewriter.TryTemplate
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.0 [2-2-2004]
 * @see org.brains2b.typewriter.TryTemplate
 */
public class TryTemplateTest extends TestCase {

	private final static String WRITE_TEST="try {\n    String value=null;\n} catch (RuntimeException rex) {\n    String value=\"exception\";\n}\n";
	private final static String WRITE_DOUBLE="try {\n    String value=null;\n} catch (RuntimeException rex) {\n    String value=\"runtime exception\";\n} catch (Exception ex) {\n    String value=\"exception\";\n}\n";
    private final static String WRITE_THROWABLE="try {\n    String value=null;\n} catch (Throwable t) {\n    String value=\"throwable\";\n}\n";
    private final static String WRITE_NOTHING="try {\n} catch (Exception ex) {\n}\n";
    
    /**
     * Constructor for TryTemplateTest.
     * @param arg0
     */
    public TryTemplateTest(String arg0) {
        super(arg0);
    }

    public static void main(String[] args) {
        junit.swingui.TestRunner.run(TryTemplateTest.class);
    }

    public void testWrite() {
    	TryTemplate t=new TryTemplate("RuntimeException");
    	t.addText("String value=null;");
    	ExceptionTemplate et=t.getDefaultException();
    	et.addText("String value=\"exception\";" );
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	t.write(pw,0);
    	pw.flush();
    	assertEquals(WRITE_TEST,sw.toString());
    }

    public void testTryTemplateClass() {
    	TryTemplate t=new TryTemplate(RuntimeException.class);
    	t.addText("String value=null;");
    	ExceptionTemplate et=t.getDefaultException();
    	et.addText("String value=\"exception\";" );
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	t.write(pw,0);
    	pw.flush();
    	assertEquals(WRITE_TEST,sw.toString());
    }

    public void testGetDefaultException() {
    	TryTemplate t=new TryTemplate(RuntimeException.class);
    	t.addText("String value=null;");
    	ExceptionTemplate et=t.getDefaultException();
    	et.addText("String value=\"runtime exception\";");
    	ExceptionTemplate e2=t.createException("Exception");
    	e2.addText("String value=\"exception\";");
    	assertEquals(et,t.getDefaultException());
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	t.write(pw,0);
    	pw.flush();
    	assertEquals(WRITE_DOUBLE,sw.toString());	
    }

    public void testCreateThrowable() {
    	TryTemplate t=new  TryTemplate(Throwable.class);
    	t.addText("String value=null;");
    	ExceptionTemplate et=t.getDefaultException();
    	et.addText("String value=\"throwable\";" );
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	t.write(pw,0);
    	pw.flush();
    	assertEquals(WRITE_THROWABLE,sw.toString());	
    	
    }

	public void testNoContent() {
    	TryTemplate t=new  TryTemplate(Exception.class);
    	//t.addText("String value=null;");
    	t.getDefaultException();
    	//et.addText("String value=\"throwable\";" );
    	StringWriter sw=new StringWriter();
    	PrintWriter pw=new PrintWriter(sw);
    	t.write(pw,0);
    	pw.flush();
    	assertEquals(WRITE_NOTHING,sw.toString());	
    	
    }
}
